#include <stdlib.h>
#include <SPI.h>
#include <SoftwareSerial.h>

// Display I/O
// Digital Inputs
#define TOP_BUTTON (17)
#define BOTTOM_BUTTON (18)

#define MISO (12)
// Digital outputs
#define SCK (13)
#define MOSI (11)
#define SW0 (17)
#define SW1 (18)
#define BUZZER (19)

#define SCAN_DELAY 1
#define SPI_ARRAY_BYTES (9)

#define CENTER_ROWS (11)
#define CENTER_COLS (11)

uint8_t eyeState = 0;

/* 
  SPI LEDs order
  Bar Array, lower right, top to bottom: 0-9
  Bar Array, upper right, top to bottom: 10-19
  Bar Array, upper left,  top to bottom: 20-29
  Bar Array, lower left,  top to bottom: 30-39

  Left eye, upper row, left to right: 40-43
  Left eye, second row, left to right: 44-47
  Left eye, third row, left to right: 48-51
  Left eye, bottom row, left to right: 52-55

  Right eye, upper row, left to right:  56-59
  Right eye, second row, left to right: 60-63
  Right eye, third row, left to right:  64-67
  Right eye, bottom row, left to right: 68-71

  72 LEDS = 9 bytes of data
*/
uint8_t eyeBarArray[SPI_ARRAY_BYTES] = { 0,0,0,0,0,0,0,0,0 }; /* Whole array to send over SPI */


/* Charlieplexed center array */
byte LED_PINS[] = { 2, 3, 4, 5, 6, 7, 8, 9, 10, 14, 15 };
const byte LED_PIN_COUNT = sizeof(LED_PINS);

struct charlie_t {
  uint8_t row;
  uint8_t col;
  uint8_t state;
};

struct charlie_t centerArray[CENTER_ROWS][CENTER_COLS];  // Array of pins combinations for each LED
uint8_t centerLevels[CENTER_COLS-1] = {3,5,6,6,7,6,6,5,5,5};

/* Input buttons */
uint8_t switchState1;
uint8_t switchState2;

uint16_t eyeDelay = 100;
uint32_t eyeTriggerTime = 0;
uint16_t barDelay = 250;
uint32_t barTriggerTime = 0;
uint16_t centerDelay = 100;
uint32_t centerTriggerTime = 0;
uint16_t debounceDelay = 50;      // the debounce time; increase if the output flickers
uint32_t debounceTime = 0;        // the last time the output pin was toggled


void SlowClock(void)
{
  digitalWrite(MOSI,LOW);
  delay(100);
  digitalWrite(SCK,LOW);
  delay(200);
  digitalWrite(SCK,HIGH);
  delay(100);

  digitalWrite(MOSI,HIGH);
  delay(100);
  digitalWrite(SCK,LOW);
  delay(200);
  digitalWrite(SCK,HIGH);
  delay(100); 
}

void displayLEDs(uint8_t ledarray[])
{
  int8_t count = 0;
  SPI.begin();
  for (count=SPI_ARRAY_BYTES-1; count>=0; count--)
  {
    SPI.transfer(ledarray[count]);
  }
  SPI.end();
}

void displayLEDs_test(void)
{
  uint8_t barArray[SPI_ARRAY_BYTES] = { 20,31,42,53,64,75,86,97,108 };
  displayLEDs(barArray);
}

/* Demo1 turns each LED on one at a time */
void demoLEDs1(void)
{
  uint8_t count,count2;
  uint8_t barArray[SPI_ARRAY_BYTES] = { 0,0,0,0,0,0,0,0,1 };

  //  Serial.println("Check all LEDs demo");

  for (count=0;count<(SPI_ARRAY_BYTES*8);count++)
  {
    for (count2=0;count2<SPI_ARRAY_BYTES;count2++)
    {
      barArray[count2]=0;
    }

    setBit(barArray, count, 1);
    displayLEDs(barArray);
    delay(50);

    /*
    Serial.print(count);
    Serial.print(":");
    printBits(barArray);
    Serial.println();
    */
  }
}

/* Random eye lights 
   Two 4x4 arrays - 32 LEDs total (bytes 0-3 in array)
*/
void randomEyes(void)
{
  uint8_t count;

  for (count=0;count<4;count++)
  {
    eyeBarArray[count]=random(0,256);
  }
}

/* Random bar lights 
   Four 10 bar arrays - 40 LEDs total (bytes 4-8 in array)
*/
void randomBars(void)
{
  uint8_t count;

  for (count=4;count<SPI_ARRAY_BYTES;count++)
  {
    eyeBarArray[count]=random(0,256);
  }
}

/* Toggle a single random bar light */
void toggleBars(void)
{
  uint8_t index, bitvalue;

  index = random(32,71);
  bitvalue = getBit(eyeBarArray, index);
  if (0==bitvalue) setBit(eyeBarArray, index, 1);
  else setBit(eyeBarArray, index, 0);
}

uint8_t getBit(uint8_t bar[], uint8_t bit_number)
{
  uint8_t bit_return =0;
  uint8_t bitSet = 1;
  uint8_t index=0;

  index = bit_number/8;
  bitSet = bitSet << (bit_number%8);

  bit_return = (bar[index] >> (bit_number%8)) & 1;
  /*
  Serial.print(index);
  Serial.print(" ");
  Serial.print(bitSet);
  Serial.print(" ");
  Serial.print(bit_return);  
  Serial.println(" ");
  */
  return bit_return;
}
 
void setBit(uint8_t bar[], uint8_t bit_number, uint8_t bit_value)
{
  uint8_t bitSet = 1;
  uint8_t index=0;

  index = bit_number/8;
  bitSet = bitSet << (bit_number%8);

  if (0==bit_value)
  {
    bitSet = ~bitSet;
    bar[index] = bar[index] & bitSet;          
  }
  else if (1==bit_value)
  {
    bar[index] = bar[index] | bitSet;
  }
}

void printBinary(uint8_t a)
{
  int8_t cnt = 0;
  for (cnt = sizeof(uint8_t) * 7; cnt >= 0; cnt--)
  {
    Serial.print((a & (1 << cnt)) >> cnt );
  }
}

void printBits(uint8_t bar[])
{
  int8_t count=0;
  for (count=SPI_ARRAY_BYTES-1; count>=0; count--)
  {
    printBinary(bar[count]);
  }
}

void setCenterState(uint8_t col, uint8_t row, uint8_t new_state)
{
  uint8_t i, j = 0;

  for (j=0; j<CENTER_COLS;j++)
  {
    for (i=0;i<CENTER_ROWS;i++)
    {
      if ((centerArray[i][j].row==row) && (centerArray[i][j].col==col))
      {
        centerArray[i][j].state = new_state;
        break;        
      }
    }
  }
}

uint8_t getCenterState(uint8_t row, uint8_t col)
{
  uint8_t i, j = 0;

  for (j=0; j<CENTER_COLS;j++)
  {
    for (i=0;i<CENTER_ROWS;i++)
    {
      if ((centerArray[i][j].row==row) && (centerArray[i][j].col==col))
      {
        return centerArray[i][j].state;
      }
    }
  }
  return 99;
}

void printCenter(void)
{
  uint8_t i, j = 0;

  Serial.println();
  for (j=0; j<CENTER_ROWS-1;j++)
  {
    Serial.println();
    Serial.print("Row");
    Serial.print(j);
    Serial.print(": ");

    for (i=0;i<CENTER_COLS-1;i++)
    {
      {
        Serial.print(getCenterState(j,i));
        Serial.print(" ");
  
      }
    }
  }
}

/* Change center positions to mimick equalizer levels */
void tweakCenter(void)
{
  uint8_t i;
  uint8_t bitState;
  uint8_t change;

  for (i=0; i<CENTER_ROWS-1;i++)
  {
    setCenterState(i,centerLevels[i],0);
    change=random(0,3);
    if (0==change)
    {
      /* Decrease */
      if (0!=centerLevels[i])
      {
        centerLevels[i]--;
      }
    }
    else if (1==change)
    {
      /* Increase */
      if (8!=centerLevels[i])
      {
        centerLevels[i]++;
      }
    }    
    setCenterState(i,centerLevels[i],1);    
  }
}


void loadCenter(void)
{
  uint8_t x,i,j=0;
  uint8_t A=0, B=1, C=2, D=3, E=4, F=5, G=6, H=7, I=8, Z=9;

  /* CH 0 High */
  centerArray[x][0].row = 99;  centerArray[x][0].col = 99;  centerArray[x][0].state = 0;
  centerArray[x][1].row = A;   centerArray[x][1].col = 0;   centerArray[x][1].state = 0;
  centerArray[x][2].row = A;   centerArray[x][2].col = 2;   centerArray[x][2].state = 0;
  centerArray[x][3].row = A;   centerArray[x][3].col = 4;   centerArray[x][3].state = 0;
  centerArray[x][4].row = A;   centerArray[x][4].col = 6;   centerArray[x][4].state = 0;
  centerArray[x][5].row = A;   centerArray[x][5].col = 8;   centerArray[x][5].state = 0;
  centerArray[x][6].row = B;   centerArray[x][6].col = 0;   centerArray[x][6].state = 0;
  centerArray[x][7].row = B;   centerArray[x][7].col = 2;   centerArray[x][7].state = 0;
  centerArray[x][8].row = B;   centerArray[x][8].col = 4;   centerArray[x][8].state = 0;
  centerArray[x][9].row = B;   centerArray[x][9].col = 6;   centerArray[x][9].state = 0;
  centerArray[x][10].row = B;  centerArray[x][10].col = 8;  centerArray[x][10].state = 0;

  /* CH 1 High */
  x=1;
  centerArray[x][0].row = A;   centerArray[x][0].col = 1;   centerArray[x][0].state = 0;
  centerArray[x][1].row = 99;  centerArray[x][1].col = 99;  centerArray[x][1].state = 0;
  centerArray[x][2].row = C;   centerArray[x][2].col = 6;   centerArray[x][2].state = 0;
  centerArray[x][3].row = C;   centerArray[x][3].col = 8;   centerArray[x][3].state = 0;
  centerArray[x][4].row = D;   centerArray[x][4].col = 0;   centerArray[x][4].state = 0;
  centerArray[x][5].row = D;   centerArray[x][5].col = 2;   centerArray[x][5].state = 0;
  centerArray[x][6].row = D;   centerArray[x][6].col = 4;   centerArray[x][6].state = 0;
  centerArray[x][7].row = D;   centerArray[x][7].col = 6;   centerArray[x][7].state = 0;
  centerArray[x][8].row = D;   centerArray[x][8].col = 8;   centerArray[x][8].state = 0;
  centerArray[x][9].row = Z;   centerArray[x][9].col = 0;   centerArray[x][9].state = 0;
  centerArray[x][10].row = Z;  centerArray[x][10].col = 1;  centerArray[x][10].state = 0;

  /* CH 2 High */
  x=2;
  centerArray[x][0].row = A;   centerArray[x][0].col = 3;   centerArray[x][0].state = 0;
  centerArray[x][1].row = C;   centerArray[x][1].col = 7;   centerArray[x][1].state = 0;
  centerArray[x][2].row = 99;  centerArray[x][2].col = 99;  centerArray[x][2].state = 0;
  centerArray[x][3].row = Z;   centerArray[x][3].col = 5;   centerArray[x][3].state = 0;
  centerArray[x][4].row = Z;   centerArray[x][4].col = 6;   centerArray[x][4].state = 0;
  centerArray[x][5].row = Z;   centerArray[x][5].col = 7;   centerArray[x][5].state = 0;
  centerArray[x][6].row = Z;   centerArray[x][6].col = 8;   centerArray[x][6].state = 0;
  centerArray[x][7].row = Z;   centerArray[x][7].col = 9;   centerArray[x][7].state = 0;
  centerArray[x][8].row = F;   centerArray[x][8].col = 0;   centerArray[x][8].state = 0;
  centerArray[x][9].row = F;   centerArray[x][9].col = 2;   centerArray[x][9].state = 0;
  centerArray[x][10].row = F;  centerArray[x][10].col = 4;  centerArray[x][10].state = 0;

  /* CH 3 High */
  x=3;
  centerArray[x][0].row = A;   centerArray[x][0].col = 5;   centerArray[x][0].state = 0;
  centerArray[x][1].row = C;   centerArray[x][1].col = 9;   centerArray[x][1].state = 0;
  centerArray[x][2].row = E;   centerArray[x][2].col = 5;   centerArray[x][2].state = 0;
  centerArray[x][3].row = 99;  centerArray[x][3].col = 99;  centerArray[x][3].state = 0;
  centerArray[x][4].row = G;   centerArray[x][4].col = 2;   centerArray[x][4].state = 0;
  centerArray[x][5].row = G;   centerArray[x][5].col = 4;   centerArray[x][5].state = 0;
  centerArray[x][6].row = G;   centerArray[x][6].col = 6;   centerArray[x][6].state = 0;
  centerArray[x][7].row = G;   centerArray[x][7].col = 8;   centerArray[x][7].state = 0;
  centerArray[x][8].row = H;   centerArray[x][8].col = 0;   centerArray[x][8].state = 0;
  centerArray[x][9].row = H;   centerArray[x][9].col = 2;   centerArray[x][9].state = 0;
  centerArray[x][10].row = H;  centerArray[x][10].col = 4;  centerArray[x][10].state = 0;

  /* CH 4 High */
  x=4;
  centerArray[x][0].row = A;   centerArray[x][0].col = 7;   centerArray[x][0].state = 0;
  centerArray[x][1].row = D;   centerArray[x][1].col = 1;   centerArray[x][1].state = 0;
  centerArray[x][2].row = E;   centerArray[x][2].col = 6;   centerArray[x][2].state = 0;
  centerArray[x][3].row = G;   centerArray[x][3].col = 3;   centerArray[x][3].state = 0;
  centerArray[x][4].row = 99;  centerArray[x][4].col = 99;  centerArray[x][4].state = 0;
  centerArray[x][5].row = I;   centerArray[x][5].col = 2;   centerArray[x][5].state = 0;
  centerArray[x][6].row = I;   centerArray[x][6].col = 4;   centerArray[x][6].state = 0;
  centerArray[x][7].row = I;   centerArray[x][7].col = 6;   centerArray[x][7].state = 0;
  centerArray[x][8].row = I;   centerArray[x][8].col = 8;   centerArray[x][8].state = 0;
  centerArray[x][9].row = H;   centerArray[x][9].col = 8;   centerArray[x][9].state = 0;
  centerArray[x][10].row = I;  centerArray[x][10].col = 0;  centerArray[x][10].state = 0;

  /* CH 5 High */
  x=5;
  centerArray[x][0].row = A;   centerArray[x][0].col = 9;   centerArray[x][0].state = 0;
  centerArray[x][1].row = D;   centerArray[x][1].col = 3;   centerArray[x][1].state = 0;
  centerArray[x][2].row = E;   centerArray[x][2].col = 7;   centerArray[x][2].state = 0;
  centerArray[x][3].row = G;   centerArray[x][3].col = 5;   centerArray[x][3].state = 0;
  centerArray[x][4].row = I;   centerArray[x][4].col = 3;   centerArray[x][4].state = 0;
  centerArray[x][5].row = 99;  centerArray[x][5].col = 99;  centerArray[x][5].state = 0;
  centerArray[x][6].row = C;   centerArray[x][6].col = 0;   centerArray[x][6].state = 0;
  centerArray[x][7].row = C;   centerArray[x][7].col = 2;   centerArray[x][7].state = 0;
  centerArray[x][8].row = C;   centerArray[x][8].col = 4;   centerArray[x][8].state = 0;
  centerArray[x][9].row = Z;   centerArray[x][9].col = 2;   centerArray[x][9].state = 0;
  centerArray[x][10].row = Z;  centerArray[x][10].col = 3;  centerArray[x][10].state = 0;

  /* CH 6 High */
  x=6;
  centerArray[x][0].row = B;   centerArray[x][0].col = 1;   centerArray[x][0].state = 0;
  centerArray[x][1].row = D;   centerArray[x][1].col = 5;   centerArray[x][1].state = 0;
  centerArray[x][2].row = E;   centerArray[x][2].col = 8;   centerArray[x][2].state = 0;
  centerArray[x][3].row = G;   centerArray[x][3].col = 7;   centerArray[x][3].state = 0;
  centerArray[x][4].row = I;   centerArray[x][4].col = 5;   centerArray[x][4].state = 0;
  centerArray[x][5].row = C;   centerArray[x][5].col = 1;   centerArray[x][5].state = 0;
  centerArray[x][6].row = 99;  centerArray[x][6].col = 99;  centerArray[x][6].state = 0;
  centerArray[x][7].row = F;   centerArray[x][7].col = 6;   centerArray[x][7].state = 0;
  centerArray[x][8].row = F;   centerArray[x][8].col = 8;   centerArray[x][8].state = 0;
  centerArray[x][9].row = G;   centerArray[x][9].col = 0;   centerArray[x][9].state = 0;
  centerArray[x][10].row = Z;  centerArray[x][10].col = 4;  centerArray[x][10].state = 0;

  /* CH 7 High */
  x=7;
  centerArray[x][0].row = B;   centerArray[x][0].col = 3;   centerArray[x][0].state = 0;
  centerArray[x][1].row = D;   centerArray[x][1].col = 7;   centerArray[x][1].state = 0;
  centerArray[x][2].row = E;   centerArray[x][2].col = 9;   centerArray[x][2].state = 0;
  centerArray[x][3].row = G;   centerArray[x][3].col = 9;   centerArray[x][3].state = 0;
  centerArray[x][4].row = I;   centerArray[x][4].col = 7;   centerArray[x][4].state = 0;
  centerArray[x][5].row = C;   centerArray[x][5].col = 3;   centerArray[x][5].state = 0;
  centerArray[x][6].row = F;   centerArray[x][6].col = 7;   centerArray[x][6].state = 0;
  centerArray[x][7].row = 99;  centerArray[x][7].col = 99;  centerArray[x][7].state = 0;
  centerArray[x][8].row = H;   centerArray[x][8].col = 6;   centerArray[x][8].state = 0;
  centerArray[x][9].row = 99;  centerArray[x][9].col = 99;  centerArray[x][9].state = 0;
  centerArray[x][10].row = 99; centerArray[x][10].col = 99; centerArray[x][10].state = 0;

  /* CH 8 High */
  x=8;
  centerArray[x][0].row = B;   centerArray[x][0].col = 5;   centerArray[x][0].state = 0;
  centerArray[x][1].row = D;   centerArray[x][1].col = 9;   centerArray[x][1].state = 0;
  centerArray[x][2].row = F;   centerArray[x][2].col = 1;   centerArray[x][2].state = 0;
  centerArray[x][3].row = H;   centerArray[x][3].col = 1;   centerArray[x][3].state = 0;
  centerArray[x][4].row = I;   centerArray[x][4].col = 9;   centerArray[x][4].state = 0;
  centerArray[x][5].row = C;   centerArray[x][5].col = 5;   centerArray[x][5].state = 0;
  centerArray[x][6].row = F;   centerArray[x][6].col = 9;   centerArray[x][6].state = 0;
  centerArray[x][7].row = H;   centerArray[x][7].col = 7;   centerArray[x][7].state = 0;
  centerArray[x][8].row = 99;  centerArray[x][8].col = 99;  centerArray[x][8].state = 0;
  centerArray[x][9].row = 99;  centerArray[x][9].col = 99;  centerArray[x][9].state = 0;
  centerArray[x][10].row = 99; centerArray[x][10].col = 99; centerArray[x][10].state = 0;

  /* CH 9 High */
  x=9;
  centerArray[x][0].row = B;   centerArray[x][0].col = 7;   centerArray[x][0].state = 0;
  centerArray[x][1].row = E;   centerArray[x][1].col = 0;   centerArray[x][1].state = 0;
  centerArray[x][2].row = F;   centerArray[x][2].col = 3;   centerArray[x][2].state = 0;
  centerArray[x][3].row = H;   centerArray[x][3].col = 3;   centerArray[x][3].state = 0;
  centerArray[x][4].row = H;   centerArray[x][4].col = 9;   centerArray[x][4].state = 0;
  centerArray[x][5].row = E;   centerArray[x][5].col = 2;   centerArray[x][5].state = 0;
  centerArray[x][6].row = G;   centerArray[x][6].col = 1;   centerArray[x][6].state = 0;
  centerArray[x][7].row = 99;  centerArray[x][7].col = 99;  centerArray[x][7].state = 0;
  centerArray[x][8].row = 99;  centerArray[x][8].col = 99;  centerArray[x][8].state = 0;
  centerArray[x][9].row = 99;  centerArray[x][9].col = 99;  centerArray[x][9].state = 0;
  centerArray[x][10].row = 99; centerArray[x][10].col = 99; centerArray[x][10].state = 0;

  /* CH 10 High */
  x=10;
  centerArray[x][0].row = B;   centerArray[x][0].col = 9;   centerArray[x][0].state = 0;
  centerArray[x][1].row = E;   centerArray[x][1].col = 1;   centerArray[x][1].state = 0;
  centerArray[x][2].row = F;   centerArray[x][2].col = 5;   centerArray[x][2].state = 0;
  centerArray[x][3].row = H;   centerArray[x][3].col = 5;   centerArray[x][3].state = 0;
  centerArray[x][4].row = I;   centerArray[x][4].col = 1;   centerArray[x][4].state = 0;
  centerArray[x][5].row = E;   centerArray[x][5].col = 3;   centerArray[x][5].state = 0;
  centerArray[x][6].row = E;   centerArray[x][6].col = 4;   centerArray[x][6].state = 0;
  centerArray[x][7].row = 99;  centerArray[x][7].col = 99;  centerArray[x][7].state = 0;
  centerArray[x][8].row = 99;  centerArray[x][8].col = 99;  centerArray[x][8].state = 0;
  centerArray[x][9].row = 99;  centerArray[x][9].col = 99;  centerArray[x][9].state = 0;
  centerArray[x][10].row = 99; centerArray[x][10].col = 99; centerArray[x][10].state = 0;

  /*
  for (j=0; j<CENTER_COLS; j++)
  {
    for (i=0; i<CENTER_ROWS; i++)
    {
      if (99!=centerArray[i][j].row)
      {
        referenceArray[centerArray[i][j].row][centerArray[i][j].col].row = i;
        referenceArray[centerArray[i][j].row][centerArray[i][j].col].col = j;
      } 
    }
  }

  for (j=0; j<CENTER_ROWS; j++)
  {
    Serial.print("Row ");
    Serial.print(j);
    Serial.print(": ");
    for (i=0; i<CENTER_COLS; i++)
    {
      Serial.print(i);
      Serial.print(" ");
    }
  }
  */

}

/* Scan through charlieplexed array */
void scanCharlie(void)
{
  uint8_t i, j=0;

  pinMode(LED_PINS[0], INPUT);
  pinMode(LED_PINS[1], INPUT);
  pinMode(LED_PINS[2], INPUT);
  pinMode(LED_PINS[3], INPUT);
  pinMode(LED_PINS[4], INPUT);
  pinMode(LED_PINS[5], INPUT);
  pinMode(LED_PINS[6], INPUT);
  pinMode(LED_PINS[7], INPUT);
  pinMode(LED_PINS[8], INPUT);
  pinMode(LED_PINS[9], INPUT);
  pinMode(LED_PINS[10], INPUT);


  for (i = 0; i < LED_PIN_COUNT; i++) {
    pinMode(LED_PINS[i], OUTPUT);
    digitalWrite(LED_PINS[i], HIGH);

    // Turn on LEDs
    for (j = 0; j < LED_PIN_COUNT; j++) {
      // Change to check the array instead
      if ((i != j) && (centerArray[i][j].row!=99))
      {
        if (centerArray[i][j].state==1)
        {
          digitalWrite(LED_PINS[j], LOW);
          pinMode(LED_PINS[j], OUTPUT);
        }
      }
    }

    delay(SCAN_DELAY);

    // Turn off LEDs    
    for (j = 0; j < LED_PIN_COUNT; j++) {
      digitalWrite(LED_PINS[j], HIGH);
      pinMode(LED_PINS[j], INPUT);
    }
  }
}

/* Scan through charlieplexed array */
void testCharlie(uint8_t i, uint8_t j)
{

  pinMode(LED_PINS[0], INPUT);
  pinMode(LED_PINS[1], INPUT);
  pinMode(LED_PINS[2], INPUT);
  pinMode(LED_PINS[3], INPUT);
  pinMode(LED_PINS[4], INPUT);
  pinMode(LED_PINS[5], INPUT);
  pinMode(LED_PINS[6], INPUT);
  pinMode(LED_PINS[7], INPUT);
  pinMode(LED_PINS[8], INPUT);
  pinMode(LED_PINS[9], INPUT);
  pinMode(LED_PINS[10], INPUT);

  pinMode(LED_PINS[i], OUTPUT);
  digitalWrite(LED_PINS[i], HIGH);
  digitalWrite(LED_PINS[j], LOW);
  pinMode(LED_PINS[j], OUTPUT);
  delay(300);

  digitalWrite(LED_PINS[i], LOW);
  digitalWrite(LED_PINS[j], HIGH);
  delay(300);
}


void setup() 
{
  
  pinMode(TOP_BUTTON, INPUT_PULLUP);
  pinMode(BOTTOM_BUTTON, INPUT_PULLUP);

  // Set up console UART
  Serial.begin(9600);
  Serial.println();
  Serial.println("Arduino Boot - WOPR ");

  pinMode(BUZZER, OUTPUT); // Set buzzer - pin 9 as an output
  tone(BUZZER, 2093); // Send 1KHz sound signal...
  delay(100);        // ...for 1 sec
  tone(BUZZER, 3135); // Send 1KHz sound signal...
  delay(100);        // ...for 1 sec
  tone(BUZZER, 4186); // Send 1KHz sound signal...
  delay(100);        // ...for 1 sec
  noTone(BUZZER);     // Stop sound...

  loadCenter();
  
//  printCenter();

  tweakCenter();
  /* Turn on all green lights */
  setCenterState(0,9,1);
  setCenterState(1,9,1);
  setCenterState(2,9,1);
  setCenterState(3,9,1);
  setCenterState(4,9,1);
  setCenterState(5,9,1);
  setCenterState(6,9,1);
  setCenterState(7,9,1);
  setCenterState(8,9,1);
  setCenterState(9,9,1);

  printCenter();

  SPI.begin();

//  displayLEDs_test();
  demoLEDs1();
  randomEyes();
  randomBars();
}

void loop()  
{

  uint8_t switchInput;
  
  if (millis() > eyeTriggerTime)
  {
    randomEyes();
    displayLEDs(eyeBarArray);    

    if (2==eyeState) 
    {
      eyeDelay = 4000;
      eyeState = 0;
    }
    else eyeDelay = 100;
    eyeState++;
    eyeTriggerTime = millis() + eyeDelay;
  }

  if (millis() > barTriggerTime)
  {
    //randomBars();
    toggleBars();
    displayLEDs(eyeBarArray);    
    barTriggerTime = millis() + barDelay;
  }

  if (millis() > centerTriggerTime)
  {
    //randomBars();
    tweakCenter();
    centerTriggerTime = millis() + centerDelay;
  }

  /* debouncing switches 
    if timer has expired,   
      when switch state is different from current state, change state immediately
      capture timer for next time state can be captured
  */
  if (millis() > debounceTime)
  {
    switchInput = digitalRead(TOP_BUTTON);
    if (switchState1 != switchInput)
    {
      switchState1 = switchInput;
      debounceTime = millis() + debounceDelay;
    }

    switchInput = digitalRead(BOTTOM_BUTTON);
    if (switchState2 != switchInput)
    {
      switchState2 = switchInput;
      debounceTime = millis() + debounceDelay;
    }
  }



/*
  SPI.begin();
  demoLEDs2();
  delay(100);
  demoLEDs2();
  delay(100);
  demoLEDs2();
  delay(4000);
  SPI.end();
*/

  SPI.end();
  scanCharlie();
}
